==> hcs-0.1.0-py2.py3-none-any.whl: azext_hcs/custom.py <==
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long

import base64
import json
import os
import requests
import sys
import webbrowser
import time

from azure.cli.command_modules.resource._client_factory import (
    _resource_client_factory, _resource_managedapps_client_factory)
from azure.mgmt.resource.managedapplications.models import Application, Plan
from msrestazure.azure_exceptions import CloudError
from knack.log import get_logger
from knack.util import CLIError

logger = get_logger(__name__)

# set up a few "constants" for use in any calls to HCP API, with ability to override for int, or local-dev.
SCHEME = "https://"
HCP_DOMAIN = os.environ.get("HCP_DOMAIN_OVERRIDE") or "api.hashicorp.cloud"
APPLICATION = "consul"
VERSION = "2020-04-13"
HCP_PLAN = os.environ.get("HCP_PLAN") or "hcs-production"

CONSUL_SERVICE_BASE_URL = "{}{}/{}/{}".format(
    SCHEME, HCP_DOMAIN, APPLICATION, VERSION)

HCS_META_BASE_URL = "https://raw.githubusercontent.com/hashicorp/cloud-hcs-meta/master/"

def check_dc_name(datacenter_name):
    if datacenter_name is None:
        datacenter_name = "dc1"
    return datacenter_name


def check_ee(external_endpoint):
    if external_endpoint is None:
        external_endpoint = "disabled"
    return external_endpoint


def check_vnet_cidr(vnet_cidr):
    if vnet_cidr is None:
        vnet_cidr = "172.25.16.0/24"
    return vnet_cidr


def check_consul_version(consul_version):
    if consul_version is None:
        consul_version = "v1.8.0"
    return consul_version


def get_plan_version():
    plan_version_url = '{}/ama-plans/version.json'.format(HCS_META_BASE_URL)
    plan_version = requests.get(plan_version_url)
    return plan_version.json()


def create_hcs(cmd, resource_group_name, name, email, location=None, datacenter_name=None,
               external_endpoint=None, vnet_cidr=None, consul_version=None, plan_name=None):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    rcf = _resource_client_factory(cmd.cli_ctx)

    resourceGroup = rcf.resource_groups.get(resource_group_name)
    if not location:
        location = resourceGroup.location

    plan_version = get_plan_version()

    parameters = {
        "clusterName": {
            "value": name
        },
        "consulDataCenter": {
            "value": check_dc_name(datacenter_name)
        },
        "consulVnetCidr": {
            "value": check_vnet_cidr(vnet_cidr)
        },
        "email": {
            "value": email
        },
        "externalEndpoint": {
            "value": check_ee(external_endpoint)
        },
        "initialConsulVersion": {
            "value": check_consul_version(consul_version)
        }
    }

    if plan_name is None:
        plan_name = "on-demand"

    application = Application(
        location=location,
        managed_resource_group_id=resourceGroup.id + '-mrg-' + name,
        kind='marketplace',
        parameters=parameters,
        plan=Plan(
            name=plan_name,
            publisher='hashicorp-4665790',
            product=HCP_PLAN,
            version=plan_version['recommended']
        )
    )
    return racf.applications.create_or_update(resource_group_name, name, application)


def delete_hcs(cmd, resource_group_name, name):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    return racf.applications.delete(resource_group_name, name)


def hcs_app_filter(app):
    return app.plan is not None and app.plan.publisher == 'hashicorp-4665790' and app.plan.product == 'hcs-production-preview'


def list_hcs(cmd, resource_group_name=None):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)

    if resource_group_name:
        applications = racf.applications.list_by_resource_group(
            resource_group_name)
    else:
        applications = racf.applications.list_by_subscription()

    return list(filter(hcs_app_filter, list(applications)))

def show_hcs(cmd, resource_group_name, name):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    rcf = _resource_client_factory(cmd.cli_ctx)

    try:
        app = racf.applications.get(resource_group_name, name)
        cluster_name = app.parameters["clusterName"]["value"]

    except AttributeError:
        raise_not_found(resource_group_name, name)

    from azure.cli.core.commands.client_factory import get_subscription_id
    subscription_id = get_subscription_id(cmd.cli_ctx)

    resource_type = 'Microsoft.Solutions/applications'
    resource_name = "{}/customconsulClusters/{}".format(name, cluster_name)
    api_version = '2018-09-01-preview'

    res = rcf.resources.get(resource_group_name,
        '',
        '',
        resource_type,
        resource_name,
        api_version,
        raw=False)

    allowed_properties = [
        "consulDatacenter",
        "consulCurrentVersion",
        "consulNumServers",
        "consulExternalEndpoint"
    ]

    properties = {}
    for key, value in res.properties.items():
        if key in allowed_properties:
            properties[key] = value

    return {
        "name": res.name,
        "properties": properties
    }

def raise_not_found(resource_group_name, name):
    msg = "Application name {} not found in resource group {}.\nPlease check your resource group and application name and try again.".format(
        name, resource_group_name)

    raise CLIError(msg)


def browse_hcs(cmd, resource_group_name, name, show_uri=False):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)

    try:
        app = racf.applications.get(resource_group_name, name)
        url = app.outputs['consul_url']['value']
    except AttributeError:
        raise_not_found(resource_group_name, name)

    if "private" in url:
        logger.warn(
            "This is a private url, you won't be able to view the UI without a VPN!")

    if show_uri:
        print('Consul url: ', url)
        sys.exit(0)

    logger.info('Consul url: ', url)
    logger.info('Opening in browser...')
    webbrowser.open(url, new=2)


def check_cluster_state(app):
    msgs = {
        "Accepted": "This application is still being provisioned.",
        "Deleting": "This application is currently being deleted.",
        "Deleted": "This application has been deleted.",
        "Failed": "This application failed to provision.",
    }
    if app.provisioning_state != "Succeeded":
        raise CLIError(msgs[app.provisioning_state])


def generate_consul_config(consulConfigValues):
    consul_config = {
        "acl": {
            "enabled": True,
            "down_policy": "async-cache",
            "default_policy": "deny",
            "tokens": {
                "agent": ""
            }
        },
        "ca_file": "./ca.pem",
        "verify_outgoing": True,
        "datacenter": consulConfigValues['datacenter'],
        "encrypt": consulConfigValues['encrypt'],
        "server": False,
        "log_level": "INFO",
        "ui": True,
        "retry_join": consulConfigValues['retry_join'],
        "auto_encrypt": {
            "tls": True
        }
    }
    consul_cfg = open("consul.json", "w")
    consul_cfg.write(json.dumps(consul_config, indent=2))
    consul_cfg.close()
    logger.info('Wrote Consul client configuration to consul.json')


def generate_ca_file(caFileValues):
    ca_file = open("ca.pem", "w")
    ca_file.write(caFileValues)
    ca_file.close()
    logger.info('Wrote Consul CA file to ca.pem')


def custom_provider_request(cmd, resource_group_name, name, endpoint, body=None):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    rcf = _resource_client_factory(cmd.cli_ctx)
    client = rcf.resources._client
    serialize = rcf.resources._serialize

    app = racf.applications.get(resource_group_name, name)

    try:
        resource_id = app.managed_resource_group_id + \
            '/providers/Microsoft.CustomProviders/resourceProviders/public'
    except AttributeError:
        raise_not_found(resource_group_name, name)

    check_cluster_state(app)

    url = client.format_url(
        '{resource_id}/{action}',
        resource_id=resource_id,
        action=serialize.url("action", endpoint, 'str'))

    query_parameters = {
        'api-version': serialize.query("api_version", "2018-09-01-preview", 'str')}

    header_parameters = {'Content-Type': 'application/json; charset=utf-8'}

    request = client.post(url, query_parameters,
                          header_parameters, body)
    response = client.send(request)

    if response.status_code not in [200, 202, 204]:
        exp = CloudError(response)
        exp.request_id = response.headers.get('x-ms-request-id')
        raise exp

    return response


def cloud_error_message(e):
    if e.inner_exception != None:
        inner = e.inner_exception
        # If this is a downstream error we assume the details is a status error
        # from our API.
        if inner.error == 'DownstreamEndpointError' and len(inner.details) == 1:
            try:
                details = inner.details[0]
                return details._message['message']
            except Exception:
                return inner._message
        else:
            return inner._message
    else:
        return e.message


def update_hcs(cmd, resource_group_name, name, consul_version):
    body = {
        "update": {
            "consulVersion": consul_version
        }
    }

    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "update", body)
        resp = json.loads(response.text)
        logger.info("Upgrade to Consul %s started." % consul_version)

        wait_operation(cmd, resource_group_name, name, resp['operation']['id'])
    except CloudError as e:
        raise CLIError(cloud_error_message(e))

    logger.info("Successfully updated cluster to consul version %s" %
                consul_version)


def wait_operation(cmd, resource_group_name, name, operation_id):
    body = {
        "operationId": operation_id,
    }
    logger.info("Waiting for operation %s to complete." % operation_id)

    # Poll the operation action every minute until the operation is completed
    try:
        while True:
            response = custom_provider_request(
                cmd, resource_group_name, name, "operation", body)

            resp = json.loads(response.text)
            operation = resp.get("operation", {})
            state = operation.get("state", "no_answer")
            if state == 'DONE':
                if is_error(operation):
                    raise CLIError("Operation failed: " +
                                   resp['result']['message'])
                return operation
            time.sleep(60)
    except CLIError as e:
        raise e
    except Exception as e:
        logger.debug(e)
        msg = "Something went wrong while waiting for operation {} to finish.".format(
            operation_id)
        raise CLIError(msg)


def is_error(operation):
    if 'result' not in operation:
        return False

    result = operation['result']
    return (isinstance(result, dict) and
            'error' in result and
            'code' in result and
            'message' in result)


def get_config(cmd, resource_group_name, name):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "config")

        resp = json.loads(response.text)
        consulConfigValues = json.loads(resp['clientConfig'])
        caFileValues = resp['caFile']

        generate_consul_config(consulConfigValues)
        generate_ca_file(caFileValues)
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def list_versions(cmd):
    try:
        endpoint = "{}/versions".format(CONSUL_SERVICE_BASE_URL)
        response = requests.get(endpoint, timeout=10)
        resp = json.loads(response.text)
        return resp
    except CloudError as ce:
        raise CLIError(cloud_error_message(ce))

def list_regions(cmd):
    supported_regions_url = '{}/regions/regions.json'.format(HCS_META_BASE_URL)
    regions_request = requests.get(supported_regions_url)
    return regions_request.json()

def create_token(cmd, resource_group_name, name, output_kubernetes_secret=False):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "createToken")

        resp = json.loads(response.text)

        if output_kubernetes_secret:
            encoded_secret = base64.b64encode(
                resp['masterToken']['secretId'].encode()).decode()
            print(token_secret_template.format(
                name=name.lower(),
                secret_id=encoded_secret))
        else:
            return resp
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def generate_kubernetes_secret(cmd, resource_group_name, name):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "config")

        resp = json.loads(response.text)
        consul_config_values = json.loads(resp['clientConfig'])
        ca_pem = resp['caFile']

        encoded_gossip_key = base64.b64encode(
            consul_config_values["encrypt"].encode()).decode()
        encoded_ca_cert = base64.b64encode(ca_pem.encode()).decode()

        print(hcs_secret_template.format(
            name=name.lower(),
            gossip_key=encoded_gossip_key,
            ca_cert=encoded_ca_cert))
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def generate_helm_values_file(cmd, resource_group_name, name, aks_cluster_name):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "config")
        resp = json.loads(response.text)
        consul_config = json.loads(resp['clientConfig'])

        from azure.cli.core.commands.client_factory import get_mgmt_service_client
        from azure.mgmt.containerservice import ContainerServiceClient

        client = get_mgmt_service_client(
            cmd.cli_ctx, ContainerServiceClient).managed_clusters
        cluster = client.get(resource_group_name, aks_cluster_name)

        print(helm_config_template.format(
            name=name.lower(),
            datacenter=consul_config['datacenter'],
            retry_join=consul_config['retry_join'],
            aks_api_server=cluster.fqdn))
    except CloudError as e:
        raise CLIError(cloud_error_message(e))

# k8s templates


token_secret_template = """apiVersion: v1
kind: Secret
metadata:
  name: {name}-bootstrap-token
type: Opaque
data:
  token: {secret_id}"""

hcs_secret_template = """apiVersion: v1
kind: Secret
metadata:
  name: {name}-hcs
type: Opaque
data:
  gossipEncryptionKey: {gossip_key}
  caCert: {ca_cert}"""

helm_config_template = """global:
  enabled: false
  name: consul
  datacenter: {datacenter}
  acls:
    manageSystemACLs: true
    bootstrapToken:
      secretName: {name}-bootstrap-token
      secretKey: token
  gossipEncryption:
    secretName: {name}-hcs
    secretKey: gossipEncryptionKey
  tls:
    enabled: true
    enableAutoEncrypt: true
    caCert:
      secretName: {name}-hcs
      secretKey: caCert
externalServers:
  enabled: true
  hosts: {retry_join}
  httpsPort: 443
  useSystemRoots: true
  k8sAuthMethodHost: https://{aks_api_server}:443
client:
  enabled: true
  # If you are using Kubenet in your AKS cluster (the default network),
  # uncomment the line below.
  # exposeGossipPorts: true
  join: {retry_join}
connectInject:
  enabled: true"""

==> hcs-0.2.0-py2.py3-none-any.whl: azext_hcs/custom.py <==
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long

import base64
import json
import os
import requests
import sys
import webbrowser
import time

from azure.cli.command_modules.resource._client_factory import (
    _resource_client_factory, _resource_managedapps_client_factory)
from azure.mgmt.resource.managedapplications.models import Application, Plan
from msrestazure.azure_exceptions import CloudError
from knack.log import get_logger
from knack.util import CLIError

logger = get_logger(__name__)

# set up a few "constants" for use in any calls to HCP API, with ability to override for int, or local-dev.
SCHEME = "https://"
HCP_DOMAIN = os.environ.get("HCP_DOMAIN_OVERRIDE") or "api.hashicorp.cloud"
APPLICATION = "consul"
VERSION = "2020-04-13"
HCP_PLAN = os.environ.get("HCP_PLAN") or "hcs-production"
HCS_PUBLISHER_ID = 'hashicorp-4665790'

CONSUL_SERVICE_BASE_URL = "{}{}/{}/{}".format(
    SCHEME, HCP_DOMAIN, APPLICATION, VERSION)

HCS_META_BASE_URL = "https://raw.githubusercontent.com/hashicorp/cloud-hcs-meta/master/"

def check_dc_name(datacenter_name):
    if datacenter_name is None:
        datacenter_name = "dc1"
    return datacenter_name


def check_ee(external_endpoint):
    if external_endpoint is None:
        external_endpoint = "disabled"
    return external_endpoint


def check_vnet_cidr(vnet_cidr):
    if vnet_cidr is None:
        vnet_cidr = "172.25.16.0/24"
    return vnet_cidr


def check_consul_version(consul_version):
    if consul_version is None:
        consul_version = "v1.8.0"
    return consul_version


def get_plan_version():
    plan_version_url = '{}/ama-plans/version.json'.format(HCS_META_BASE_URL)
    plan_version = requests.get(plan_version_url)
    return plan_version.json()


def create_hcs(cmd, resource_group_name, name, email, location=None, datacenter_name=None,
               external_endpoint=None, vnet_cidr=None, consul_version=None, plan_name=None):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    rcf = _resource_client_factory(cmd.cli_ctx)

    resourceGroup = rcf.resource_groups.get(resource_group_name)
    if not location:
        location = resourceGroup.location

    plan_version = get_plan_version()

    parameters = {
        "clusterName": {
            "value": name
        },
        "consulDataCenter": {
            "value": check_dc_name(datacenter_name)
        },
        "consulVnetCidr": {
            "value": check_vnet_cidr(vnet_cidr)
        },
        "email": {
            "value": email
        },
        "externalEndpoint": {
            "value": check_ee(external_endpoint)
        },
        "initialConsulVersion": {
            "value": check_consul_version(consul_version)
        }
    }

    if plan_name is None:
        plan_name = "on-demand"

    application = Application(
        location=location,
        managed_resource_group_id=resourceGroup.id + '-mrg-' + name,
        kind='marketplace',
        parameters=parameters,
        plan=Plan(
            name=plan_name,
            publisher=HCS_PUBLISHER_ID,
            product=HCP_PLAN,
            version=plan_version['recommended']
        )
    )
    return racf.applications.create_or_update(resource_group_name, name, application)


def delete_hcs(cmd, resource_group_name, name):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    return racf.applications.delete(resource_group_name, name)


def hcs_app_filter(app):
    return app.plan is not None and app.plan.publisher == HCS_PUBLISHER_ID and app.plan.product == HCP_PLAN


def list_hcs(cmd, resource_group_name=None):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)

    if resource_group_name:
        applications = racf.applications.list_by_resource_group(
            resource_group_name)
    else:
        applications = racf.applications.list_by_subscription()

    return list(filter(hcs_app_filter, list(applications)))

def show_hcs(cmd, resource_group_name, name):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    rcf = _resource_client_factory(cmd.cli_ctx)

    try:
        app = racf.applications.get(resource_group_name, name)
        cluster_name = app.parameters["clusterName"]["value"]

    except AttributeError:
        raise_not_found(resource_group_name, name)

    from azure.cli.core.commands.client_factory import get_subscription_id
    subscription_id = get_subscription_id(cmd.cli_ctx)

    resource_type = 'Microsoft.Solutions/applications'
    resource_name = "{}/customconsulClusters/{}".format(name, cluster_name)
    api_version = '2018-09-01-preview'

    res = rcf.resources.get(resource_group_name,
        '',
        '',
        resource_type,
        resource_name,
        api_version,
        raw=False)

    allowed_properties = [
        "consulDatacenter",
        "consulCurrentVersion",
        "consulNumServers",
        "consulExternalEndpoint"
    ]

    properties = {}
    for key, value in res.properties.items():
        if key in allowed_properties:
            properties[key] = value

    return {
        "name": res.name,
        "properties": properties
    }

def raise_not_found(resource_group_name, name):
    msg = "Application name {} not found in resource group {}.\nPlease check your resource group and application name and try again.".format(
        name, resource_group_name)

    raise CLIError(msg)


def browse_hcs(cmd, resource_group_name, name, show_uri=False):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)

    try:
        app = racf.applications.get(resource_group_name, name)
        url = app.outputs['consul_url']['value']
    except AttributeError:
        raise_not_found(resource_group_name, name)

    if "private" in url:
        logger.warn(
            "This is a private url, you won't be able to view the UI without a VPN!")

    if show_uri:
        print('Consul url: ', url)
        sys.exit(0)

    logger.info('Consul url: ', url)
    logger.info('Opening in browser...')
    webbrowser.open(url, new=2)


def check_cluster_state(app):
    msgs = {
        "Accepted": "This application is still being provisioned.",
        "Deleting": "This application is currently being deleted.",
        "Deleted": "This application has been deleted.",
        "Failed": "This application failed to provision.",
    }
    if app.provisioning_state != "Succeeded":
        raise CLIError(msgs[app.provisioning_state])


def generate_consul_config(consulConfigValues):
    consul_config = {
        "acl": {
            "enabled": True,
            "down_policy": "async-cache",
            "default_policy": "deny",
            "tokens": {
                "agent": ""
            }
        },
        "ca_file": "./ca.pem",
        "verify_outgoing": True,
        "datacenter": consulConfigValues['datacenter'],
        "encrypt": consulConfigValues['encrypt'],
        "server": False,
        "log_level": "INFO",
        "ui": True,
        "retry_join": consulConfigValues['retry_join'],
        "auto_encrypt": {
            "tls": True
        }
    }
    consul_cfg = open("consul.json", "w")
    consul_cfg.write(json.dumps(consul_config, indent=2))
    consul_cfg.close()
    logger.info('Wrote Consul client configuration to consul.json')


def generate_ca_file(caFileValues):
    ca_file = open("ca.pem", "w")
    ca_file.write(caFileValues)
    ca_file.close()
    logger.info('Wrote Consul CA file to ca.pem')


def custom_provider_request(cmd, resource_group_name, name, endpoint, body=None):
    racf = _resource_managedapps_client_factory(cmd.cli_ctx)
    rcf = _resource_client_factory(cmd.cli_ctx)
    client = rcf.resources._client
    serialize = rcf.resources._serialize

    app = racf.applications.get(resource_group_name, name)

    try:
        resource_id = app.managed_resource_group_id + \
            '/providers/Microsoft.CustomProviders/resourceProviders/public'
    except AttributeError:
        raise_not_found(resource_group_name, name)

    check_cluster_state(app)

    url = client.format_url(
        '{resource_id}/{action}',
        resource_id=resource_id,
        action=serialize.url("action", endpoint, 'str'))

    query_parameters = {
        'api-version': serialize.query("api_version", "2018-09-01-preview", 'str')}

    header_parameters = {'Content-Type': 'application/json; charset=utf-8'}

    request = client.post(url, query_parameters,
                          header_parameters, body)
    response = client.send(request)

    if response.status_code not in [200, 202, 204]:
        exp = CloudError(response)
        exp.request_id = response.headers.get('x-ms-request-id')
        raise exp

    return response


def cloud_error_message(e):
    if e.inner_exception != None:
        inner = e.inner_exception
        # If this is a downstream error we assume the details is a status error
        # from our API.
        if inner.error == 'DownstreamEndpointError' and len(inner.details) == 1:
            try:
                details = inner.details[0]
                return details._message['message']
            except Exception:
                return inner._message
        else:
            return inner._message
    else:
        return e.message


def update_hcs(cmd, resource_group_name, name, consul_version):
    body = {
        "update": {
            "consulVersion": consul_version
        }
    }

    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "update", body)
        resp = json.loads(response.text)
        logger.info("Upgrade to Consul %s started." % consul_version)

        wait_operation(cmd, resource_group_name, name, resp['operation']['id'])
    except CloudError as e:
        raise CLIError(cloud_error_message(e))

    logger.info("Successfully updated cluster to consul version %s" %
                consul_version)


def wait_operation(cmd, resource_group_name, name, operation_id):
    body = {
        "operationId": operation_id,
    }
    logger.info("Waiting for operation %s to complete." % operation_id)

    # Poll the operation action every minute until the operation is completed
    try:
        while True:
            response = custom_provider_request(
                cmd, resource_group_name, name, "operation", body)

            resp = json.loads(response.text)
            operation = resp.get("operation", {})
            state = operation.get("state", "no_answer")
            if state == 'DONE':
                if is_error(operation):
                    raise CLIError("Operation failed: " +
                                   resp['result']['message'])
                return operation
            time.sleep(60)
    except CLIError as e:
        raise e
    except Exception as e:
        logger.debug(e)
        msg = "Something went wrong while waiting for operation {} to finish.".format(
            operation_id)
        raise CLIError(msg)


def is_error(operation):
    if 'result' not in operation:
        return False

    result = operation['result']
    return (isinstance(result, dict) and
            'error' in result and
            'code' in result and
            'message' in result)


def get_config(cmd, resource_group_name, name):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "config")

        resp = json.loads(response.text)
        consulConfigValues = json.loads(resp['clientConfig'])
        caFileValues = resp['caFile']

        generate_consul_config(consulConfigValues)
        generate_ca_file(caFileValues)
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def list_versions(cmd):
    try:
        endpoint = "{}/versions".format(CONSUL_SERVICE_BASE_URL)
        response = requests.get(endpoint, timeout=10)
        resp = json.loads(response.text)
        return resp
    except CloudError as ce:
        raise CLIError(cloud_error_message(ce))

def list_regions(cmd):
    supported_regions_url = '{}/regions/regions.json'.format(HCS_META_BASE_URL)
    regions_request = requests.get(supported_regions_url)
    return regions_request.json()

def create_token(cmd, resource_group_name, name, output_kubernetes_secret=False):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "createToken")

        resp = json.loads(response.text)

        if output_kubernetes_secret:
            encoded_secret = base64.b64encode(
                resp['masterToken']['secretId'].encode()).decode()
            print(token_secret_template.format(
                name=name.lower(),
                secret_id=encoded_secret))
        else:
            return resp
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def get_billing_info(cmd, resource_group_name, name):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "billingInfo")

        return json.loads(response.text)
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def get_billing_report(cmd, resource_group_name, name, month):
    body = {
         "month": month
    }

    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "billingReport", body)

        return json.loads(response.text)
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def generate_kubernetes_secret(cmd, resource_group_name, name):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "config")

        resp = json.loads(response.text)
        consul_config_values = json.loads(resp['clientConfig'])
        ca_pem = resp['caFile']

        encoded_gossip_key = base64.b64encode(
            consul_config_values["encrypt"].encode()).decode()
        encoded_ca_cert = base64.b64encode(ca_pem.encode()).decode()

        print(hcs_secret_template.format(
            name=name.lower(),
            gossip_key=encoded_gossip_key,
            ca_cert=encoded_ca_cert))
    except CloudError as e:
        raise CLIError(cloud_error_message(e))


def generate_helm_values_file(cmd, resource_group_name, name, aks_cluster_name):
    try:
        response = custom_provider_request(
            cmd, resource_group_name, name, "config")
        resp = json.loads(response.text)
        consul_config = json.loads(resp['clientConfig'])

        from azure.cli.core.commands.client_factory import get_mgmt_service_client
        from azure.mgmt.containerservice import ContainerServiceClient

        client = get_mgmt_service_client(
            cmd.cli_ctx, ContainerServiceClient).managed_clusters
        cluster = client.get(resource_group_name, aks_cluster_name)

        print(helm_config_template.format(
            name=name.lower(),
            datacenter=consul_config['datacenter'],
            retry_join=consul_config['retry_join'],
            aks_api_server=cluster.fqdn))
    except CloudError as e:
        raise CLIError(cloud_error_message(e))

# k8s templates


token_secret_template = """apiVersion: v1
kind: Secret
metadata:
  name: {name}-bootstrap-token
type: Opaque
data:
  token: {secret_id}"""

hcs_secret_template = """apiVersion: v1
kind: Secret
metadata:
  name: {name}-hcs
type: Opaque
data:
  gossipEncryptionKey: {gossip_key}
  caCert: {ca_cert}"""

helm_config_template = """global:
  enabled: false
  name: consul
  datacenter: {datacenter}
  acls:
    manageSystemACLs: true
    bootstrapToken:
      secretName: {name}-bootstrap-token
      secretKey: token
  gossipEncryption:
    secretName: {name}-hcs
    secretKey: gossipEncryptionKey
  tls:
    enabled: true
    enableAutoEncrypt: true
    caCert:
      secretName: {name}-hcs
      secretKey: caCert
externalServers:
  enabled: true
  hosts: {retry_join}
  httpsPort: 443
  useSystemRoots: true
  k8sAuthMethodHost: https://{aks_api_server}:443
client:
  enabled: true
  # If you are using Kubenet in your AKS cluster (the default network),
  # uncomment the line below.
  # exposeGossipPorts: true
  join: {retry_join}
connectInject:
  enabled: true"""
